/*
AC Light Control
Uses up and down buttons to set levels
makes use of a timer interrupt to set the level of dimming
*/
#include <SPI.h>
#include <MySensor.h>  
#include <TimerOne.h>
//#include <PinChangeInt.h>
#include <EEPROM.h>
#include <avr/interrupt.h>


#define DEBUG_ON

//
#ifdef DEBUG_ON
#define DEBUG_PRINT(x)   Serial.print(x)
#define DEBUG_PRINTLN(x) Serial.println(x)
#define SERIAL_START(x)  Serial.begin(x)
#else
#define DEBUG_PRINT(x)
#define DEBUG_PRINTLN(x)
#define SERIAL_START(x)
#endif
//
#define LIGHT_OFF 0
#define LIGHT_ON 1


//External Pin configuration
#define ZERO_CROSS 3  // PD3 Pin for zero cross detection
#define RESET_CONF 4  // PD4 Pin for reset of config and clearing node Id
#define DIM1_BUTTON 15 //PC1 pin for External control of channel 1 of the dimmer
#define DIM2_BUTTON 16 //PC2 pin for External control of channel 2 of the dimmer
#define AMPERE_VALUE 17 //PC3 pin for measuring used ampere in the dimmer and its channels
#define AMPERE_CTRL 19 //PC5 pin for turning on off ampere measurement logic(components)
#define DIMMER1_CTRL 5 //PD5 pin for controlling dimmer 1 triac
#define DIMMER2_CTRL 6 //PD6 pin for controlling dimmer 2 triac

//Mysensors definitions
#define SKETCH_NAME "AC Dimmer control"
#define SKETCH_VERSION "1.0"

#define DIMMER_START_ADDRESS 1
#define NUMBER_OF_DIMMERS 2
#define DIMMER1 DIMMER_START_ADDRESS      //Defines number for mysensor channel
#define DIMMER2 (DIMMER_START_ADDRESS+1)  //Defines number for mysensor channel
MySensor gw;
MyMessage dimmerMsg(DIMMER1, V_DIMMER);
MyMessage lightMsg(DIMMER1, V_LIGHT);
MyMessage powerMsg(DIMMER1,V_WATT);

//EEPROM ADRESS definition
//Check the start adress EEPROM_LOCAL_CONFIG_ADDRESS before compilation, should be valid, but can eventually change with updates in mysensor libraries
#define EEPROM_PARAM_STORED       EEPROM_LOCAL_CONFIG_ADDRESS   //Address from Mysensors.h
#define EEPROM_ADDRESS_DIM1_LEVEL         (EEPROM_PARAM_STORED + 4)
#define EEPROM_ADDRESS_DIM1_ONOFF         (EEPROM_ADDRESS_DIM1_LEVEL + 8)
#define EEPROM_ADDRESS_DIM2_LEVEL         (EEPROM_ADDRESS_DIM1_ONOFF + 4)
#define EEPROM_ADDRESS_DIM2_ONOFF         (EEPROM_ADDRESS_DIM2_LEVEL + 8)

//Adding addresses so it fits the rest of the program
typedef struct{
   int onOffAddress;
   int levelAddress;
}eepromAddressClass;

eepromAddressClass dim1EepromAddr, dim2EepromAddr;
eepromAddressClass eepromAddresses[NUMBER_OF_DIMMERS] = {dim1EepromAddr, dim2EepromAddr};




//Dimmer states definition
typedef enum{
  DIM_OFF = 0,
  DIM_ON,
  DIM_WAIT,
  DIM_FADE,
  DIM_END
}dimmerStates;

dimmerStates dim1State, dim1StateOld;

#define DIM_FADE_DELAY 20  //Delay in ms for each percentage fade up/down (10ms = 1s full-range dim)
#define DIM_LEVEL_DEFAULT 50

//Interrupts
volatile int dim1Counter, dim2Counter = 0;      // Variable to use as a counter of dimming steps. It is volatile since it is passed between interrupts
volatile boolean zero_cross1, zero_cross2 = 0;  // Flag to indicate we have crossed zero

volatile uint16_t voMeasured = 0;  //measured power of the dimmer 

int freqStep = 100;                             // This is the delay-per-brightness step in microseconds. It allows for 100 steps within a 50 Hz system Needs to be changed if 60 Hz                                             

//Button definitions
#define BUTTON_DEBOUNCE 100
#define RESTORE_BUTTON_TIME 50        //Connected to freqStep equals RESTORE_BUTTON_TIME * freqStep seconds
#define BUTTON_TIME_OUT_2CLICK 2000   //time out in millis before 2 click
#define BUTTON_TIME_OUT_FADE 20000    //time out in millis before leaving fade state

int countButtonTimer = 0;
volatile long unsigned eventTimeButton = 0;
volatile long unsigned previousEventTimeButton = 0;

//Dimmer Class definition
typedef struct{
  int eepromAddressOnOff;
  int eepromAddressLevel;
  int onOff;                
  int level;                
  dimmerStates state;       
  dimmerStates stateOld;    
  int levelOld;
  int levelNew;             
  int fadeStep;             
  volatile long unsigned eventButtonTime;
  volatile long unsigned lastChangeButtonTime;
  volatile boolean buttonFlag;
  volatile int buttonNew;
  volatile int buttonOld;

}dimmerClass;

dimmerClass dimmer1Class, dimmer2Class;
dimmerClass dimmers[NUMBER_OF_DIMMERS] = {dimmer1Class, dimmer2Class};



/*------------------------------------------------------------------*/
/*EEPROM Address definition                                         */
/*                                                                  */
/*------------------------------------------------------------------*/
void eepromAddressDefinition(){
  DEBUG_PRINTLN(F("EEPROM address definition start"));
  eepromAddresses[DIMMER1-DIMMER_START_ADDRESS].onOffAddress = EEPROM_ADDRESS_DIM1_ONOFF;
  eepromAddresses[DIMMER1-DIMMER_START_ADDRESS].levelAddress = EEPROM_ADDRESS_DIM1_LEVEL;
  eepromAddresses[DIMMER2-DIMMER_START_ADDRESS].onOffAddress = EEPROM_ADDRESS_DIM2_ONOFF;
  eepromAddresses[DIMMER2-DIMMER_START_ADDRESS].levelAddress = EEPROM_ADDRESS_DIM2_LEVEL;
  DEBUG_PRINTLN(F("EEPROM address definition end"));
}

/*------------------------------------------------------------------*/
/*Setup                                                             */
/*                                                                  */
/*------------------------------------------------------------------*/
void setup() {  // Begin setup
  //clearEeprom();    //Just in case you want to clear EEPROM, comment out, run and then comment.
  Serial.begin(115200);
  /// - Setup Mysensors
 
  gw.begin( incomingMessage,  AUTO, true);
  Serial.println("---------------");
  Serial.println( "Sensor setup start" ); 

  //Setup AC PIN
  pinMode(DIMMER1_CTRL, OUTPUT);                          // Set the Triac pin as output
  pinMode(DIMMER2_CTRL, OUTPUT);                          // Set the Triac pin as output


  //pinMode(AMPERE_CTRL, OUTPUT);                          // Set the AC measurement control/enable pin as output
  //digitalWrite(AMPERE_CTRL, HIGH);          //Turn on power to power measurement


  //Setup zero cross pin
  pinMode(ZERO_CROSS, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(ZERO_CROSS), zero_cross_detect, RISING);

  //define the reset conf pin
  pinMode(RESET_CONF, INPUT); //allready pull up on board digitalWrite(RESET_CONF,HIGH);

  //define Switch pins
  pinMode(DIM1_BUTTON, INPUT); //allready pull up on board digitalWrite(DIM1_BUTTON,HIGH);
  pinMode(DIM2_BUTTON, INPUT); //allready pull up on board digitalWrite(DIM2_BUTTON,HIGH);

  cli(); //Turn off all interrupts
  //Initiates button interrupts
  PCICR |= 0b00000110; // 0bXXXXXDCB, Enables Ports C and D Pin Change Interrupts
  PCMSK1 |= 0b00000110; // PCINT8 to PCINT14, PC0 to PC6 (enable PC1, PC2)
  PCMSK2 |= 0b00010000; // PCINT16 to PCINT23, PD0 to PD7 (enable PD4)
  sei(); //turn on all interrupts

  //Initialize timer interrupt 
  Timer1.initialize(freqStep);                      // Initialize TimerOne library for the freq we need
  Timer1.attachInterrupt(timer1Callback, freqStep);      // Go to timer1Callback procedure every 75 uS (50Hz)  or 65 uS (60Hz) 
  // Use the TimerOne Library to attach an interrupt


  //writes default parameters to eeprom and reads user set parameters from EEPROM
  WriteDefaultParametersEEPROM();

  eepromAddressDefinition();
  presentSensors(); //present sensors to GW
  setupDimmers(DIMMER_START_ADDRESS, NUMBER_OF_DIMMERS);
  
// Retreive our last dim levels from the eprom
  sendUpdateToControllerAll(DIMMER_START_ADDRESS, NUMBER_OF_DIMMERS);
  
  Serial.println("Sensor setup end");
  Serial.println("---------------");  
}


/*------------------------------------------------------------------*/
/*Setup dimmers                                                      */
/*                                                                  */
/*------------------------------------------------------------------*/

void setupDimmers(int startAddress, int numberOfSensors){
  DEBUG_PRINTLN(F("Dimmer mult setup Start"));
  for (int ii = 0 ; ii < numberOfSensors ; ii++) {
    setupDimmer(ii+startAddress);
  }
  DEBUG_PRINTLN(F("Dimmer mult setup end"));
}

/*------------------------------------------------------------------*/
/*Setup dimmer                                                      */
/*                                                                  */
/*------------------------------------------------------------------*/

void setupDimmer(int dimmerToSetup){
  DEBUG_PRINT(F("Dimmer setup start, sensor: "));
  DEBUG_PRINTLN(dimmerToSetup);
  dimmerToSetup = dimmerToSetup - DIMMER_START_ADDRESS;
  
  dimmers[dimmerToSetup].eepromAddressOnOff = eepromAddresses[dimmerToSetup].onOffAddress;
  dimmers[dimmerToSetup].eepromAddressLevel = eepromAddresses[dimmerToSetup].levelAddress; 
  EEPROM.get(dimmers[dimmerToSetup].eepromAddressOnOff, dimmers[dimmerToSetup].onOff);
  EEPROM.get(dimmers[dimmerToSetup].eepromAddressLevel, dimmers[dimmerToSetup].level);
  dimmers[dimmerToSetup].state = DIM_OFF;
  dimmers[dimmerToSetup].stateOld = DIM_OFF;
  dimmers[dimmerToSetup].levelOld = dimmers[dimmerToSetup].level;
  dimmers[dimmerToSetup].levelNew = dimmers[dimmerToSetup].level;
  dimmers[dimmerToSetup].fadeStep = 1;
  dimmers[dimmerToSetup].eventButtonTime = 0;
  dimmers[dimmerToSetup].lastChangeButtonTime = 0;
  dimmers[dimmerToSetup].buttonFlag = false;
  dimmers[dimmerToSetup].buttonNew = 1;
  dimmers[dimmerToSetup].buttonOld = 1;  
}


/*------------------------------------------------------------------*/
/*present sensors and channels to GW                                */
/*                                                                  */
/*------------------------------------------------------------------*/
void presentSensors(){
   /*----------------*/
   /*Notify gateway*/
  DEBUG_PRINTLN(F("Sensor Presentation Start"));
  // Send the sketch version information to the gateway and Controller
  gw.sendSketchInfo(SKETCH_NAME, SKETCH_VERSION);

  // Register the LED Dimmable Light with the gateway
  gw.present( DIMMER1, S_DIMMER );
  gw.present( DIMMER2, S_DIMMER );
  gw.present( DIMMER1, S_LIGHT);
  gw.present( DIMMER2, S_LIGHT);

  DEBUG_PRINTLN(F("Sensor Presentation Complete")); 
}


/*------------------------------------------------------------------*/
/*ISR Routine Zero cross detect                                     */
/*                                                                  */
/*------------------------------------------------------------------*/
void zero_cross_detect() {    
  zero_cross1 = true;               // set flag for timer1Callback function that a zero cross has occured
  zero_cross2 = true;               // set flag for timer1Callback function that a zero cross has occured
  dim1Counter=100-dimmers[DIMMER1-DIMMER_START_ADDRESS].level;;             // stepcounter to dimLevel.... as we start a new cycle
  dim2Counter=100-dimmers[DIMMER2-DIMMER_START_ADDRESS].level;              // stepcounter to dimLevel.... as we start a new cycle
  digitalWrite(DIMMER1_CTRL, LOW);
  digitalWrite(DIMMER2_CTRL, LOW);
}  

/*------------------------------------------------------------------*/
/*ISR dimmer button interrupts for ON/OFF and change dim level      */
/*                                                                  */
/*------------------------------------------------------------------*/
ISR(PCINT1_vect)
{
  eventTimeButton = millis();
  
  dimmers[DIMMER1-DIMMER_START_ADDRESS].buttonNew = digitalRead(DIM1_BUTTON);
  dimmers[DIMMER2-DIMMER_START_ADDRESS].buttonNew = digitalRead(DIM2_BUTTON);

  DEBUG_PRINT(F("Delta (ms): ")); 
  DEBUG_PRINTLN((eventTimeButton-previousEventTimeButton)); 

  if(eventTimeButton-previousEventTimeButton > BUTTON_DEBOUNCE){
    previousEventTimeButton = eventTimeButton;
    
    if(dimmers[DIMMER1-DIMMER_START_ADDRESS].buttonNew != dimmers[DIMMER1-DIMMER_START_ADDRESS].buttonOld) {
      dimmers[DIMMER1-DIMMER_START_ADDRESS].eventButtonTime = eventTimeButton;
      DEBUG_PRINTLN(F("Button 1 interrupt"));
      dimmers[DIMMER1-DIMMER_START_ADDRESS].buttonFlag = true;
    }
    
    if(dimmers[DIMMER2-DIMMER_START_ADDRESS].buttonNew != dimmers[DIMMER2-DIMMER_START_ADDRESS].buttonOld) {
      dimmers[DIMMER2-DIMMER_START_ADDRESS].eventButtonTime = eventTimeButton;
      DEBUG_PRINTLN(F("Button 2 interrupt"));
      dimmers[DIMMER2-DIMMER_START_ADDRESS].buttonFlag = true;
    } 
  }
  dimmers[DIMMER1-DIMMER_START_ADDRESS].buttonOld = dimmers[DIMMER1-DIMMER_START_ADDRESS].buttonNew;
  dimmers[DIMMER2-DIMMER_START_ADDRESS].buttonOld = dimmers[DIMMER2-DIMMER_START_ADDRESS].buttonNew;  
}


/*------------------------------------------------------------------*/
/*ISR Restore dafault pin interrupt                                 */
/*                                                                  */
/*------------------------------------------------------------------*/
ISR(PCINT2_vect)
{
  if(digitalRead(RESET_CONF)==0){
    DEBUG_PRINT(F("Restore dafaults"));
  }

  if (digitalRead(RESET_CONF)==0){
    DEBUG_PRINTLN(F("Button restore pressed"));
    countButtonTimer = 0;
  }
  else{
    DEBUG_PRINTLN(F("Button restore released"));
    DEBUG_PRINT(F("button counter: "));
    DEBUG_PRINTLN(countButtonTimer);
    if (countButtonTimer >= RESTORE_BUTTON_TIME){
      DEBUG_PRINTLN(F("Reset initiated"));
      //clearEeprom(); //clear EEPROM
      //WriteDefaultParametersEEPROM(); //writes back default parameters to EEPROM
      /////////////Think this through more should not reinitiate whole thing only default parameters
      //setupDimmers(DIMMER_START_ADDRESS, NUMBER_OF_DIMMERS); 
      
    }
  }
  
}


/*------------------------------------------------------------------*/
/*ISR Restore dafault pin interrupt                                 */
/* Turn on the TRIAC at the appropriate time 
 * We arrive here every 100 (TBD) uS 
 * First check if a flag has been set
 * Then check if the counter 'dim1Counter, dim2Counter' has reached the dimming level
 * if so.... switch on the TRIAC and reset the counter
 */
/*------------------------------------------------------------------*/
void timer1Callback() {
  countButtonTimer++;
  //voMeasured = analogRead(AMPERE_VALUE);// Get Ampere value
  if(zero_cross1 == true & dimmers[DIMMER1-DIMMER_START_ADDRESS].onOff == LIGHT_ON) {     
    if(dim1Counter<=0 ) {                     
      digitalWrite(DIMMER1_CTRL, HIGH);  // turn on light                        
      zero_cross1=false;    // reset zero cross detection flag
    } 
    else {
      dim1Counter--;  // increment time step counter                     
    }                                
  }

  if(zero_cross2 == true & dimmers[DIMMER2-DIMMER_START_ADDRESS].onOff == LIGHT_ON) {              
    if(dim2Counter<=0 ) {                     
      digitalWrite(DIMMER2_CTRL, HIGH);  // turn on light                        
      zero_cross2=false;    // reset zero cross detection flag
    } 
    else {
      dim2Counter--;  // decrement time step counter                     
    }                                
  }
       
}

/*------------------------------------------------------------------*/
/*Main program loop                                                 */
/*                                                                  */
/*------------------------------------------------------------------*/
void loop() {  
  gw.process();
  stateUpdate(DIMMER1);
  stateUpdate(DIMMER2);  
}

/*------------------------------------------------------------------*/
/*Send update to controller                                         */
/*                                                                  */
/*------------------------------------------------------------------*/
void sendUpdateToControllerAll(int startAddress, int numberOfSensors){
  for (int ii = 0 ; ii < numberOfSensors; ii++){ 
    sendUpdateToController(ii+startAddress);
  }
}


/*------------------------------------------------------------------*/
/*Send update to controller                                         */
/*                                                                  */
/*------------------------------------------------------------------*/
void sendUpdateToController(int dimmerToSendTo){
  int dimmerToSendToAdjusted = dimmerToSendTo - DIMMER_START_ADDRESS;
  
  DEBUG_PRINTLN(F("Send update to controller: "));
  DEBUG_PRINT(F("Dimmer "));
  DEBUG_PRINT(dimmerToSendTo);
  DEBUG_PRINT(F(": "));
  DEBUG_PRINTLN(dimmers[dimmerToSendToAdjusted].level);  
  gw.send(dimmerMsg.setSensor(dimmerToSendTo).set(dimmers[dimmerToSendToAdjusted].level));  
  DEBUG_PRINT(F("Dimmer status (0=OFF, 1=ON): "));
  DEBUG_PRINTLN(dimmers[dimmerToSendToAdjusted].onOff);
  gw.send(lightMsg.setSensor(dimmerToSendTo).set(dimmers[dimmerToSendToAdjusted].onOff));

}


/*------------------------------------------------------------------*/
/*State update                                                      */
/*                                                                  */
/*------------------------------------------------------------------*/
                            
void stateUpdate(int dimmerToUpdate){
  dimmerToUpdate = dimmerToUpdate - DIMMER_START_ADDRESS;
  
  switch(dimmers[dimmerToUpdate].state){
      case DIM_OFF:
        dimmers[dimmerToUpdate].stateOld = dimmers[dimmerToUpdate].state;
        if (dimmers[dimmerToUpdate].level != dimmers[dimmerToUpdate].levelNew){
          DEBUG_PRINTLN(F("Changing value"));
          dimmers[dimmerToUpdate].level = dimmers[dimmerToUpdate].levelNew;                    
          EEPROM.put(dimmers[dimmerToUpdate].eepromAddressLevel, dimmers[dimmerToUpdate].level);
          sendUpdateToController(dimmerToUpdate + DIMMER_START_ADDRESS);
        }

        if(dimmers[dimmerToUpdate].onOff == 1 || dimmers[dimmerToUpdate].buttonFlag == true){
          if (dimmers[dimmerToUpdate].buttonFlag == true){
            DEBUG_PRINTLN(F("Button pressed"));
          }
          DEBUG_PRINTLN(F("--------------"));
          DEBUG_PRINTLN(F("Leaving from OFF to ON"));
          fadeToLevelDimmer(dimmerToUpdate+DIMMER_START_ADDRESS, 0, dimmers[dimmerToUpdate].level); 
          dimmers[dimmerToUpdate].onOff = 1;
          EEPROM.put(dimmers[dimmerToUpdate].eepromAddressOnOff, dimmers[dimmerToUpdate].onOff);    
          dimmers[dimmerToUpdate].state = DIM_ON;
          sendUpdateToController(dimmerToUpdate + DIMMER_START_ADDRESS);
          dimmers[dimmerToUpdate].buttonFlag = false;  //Must be last! Else timing issue. Reset button flag in case it was a button that triggered the leaving
        }
      break;

      case DIM_ON:
        dimmers[dimmerToUpdate].stateOld = dimmers[dimmerToUpdate].state;
        if (dimmers[dimmerToUpdate].level != dimmers[dimmerToUpdate].levelNew){
          DEBUG_PRINTLN(F("Fading"));
          dimmers[dimmerToUpdate].level = fadeToLevelDimmer(dimmerToUpdate+DIMMER_START_ADDRESS, dimmers[dimmerToUpdate].level, dimmers[dimmerToUpdate].levelNew);
          //dimmers[dimmerToUpdate].level = fadeToLevel( dimmers[dimmerToUpdate].level, dimmers[dimmerToUpdate].levelNew ); 
          EEPROM.put(dimmers[dimmerToUpdate].eepromAddressLevel, dimmers[dimmerToUpdate].level);            
          sendUpdateToController(dimmerToUpdate + DIMMER_START_ADDRESS);
        }

        if (dimmers[dimmerToUpdate].buttonFlag == true && dimmers[dimmerToUpdate].buttonNew == 1){  //Wait for rising edge
          DEBUG_PRINTLN(F("Button pressed"));
          DEBUG_PRINTLN(F("--------------"));
          DEBUG_PRINTLN(F("Leaving from ON to WAIT"));
          dimmers[dimmerToUpdate].state = DIM_WAIT;
          dimmers[dimmerToUpdate].buttonFlag = false;  //Must be last or timing issue, should trigger on both 1 to 0 and 0 to 1
        }
        
        if(dimmers[dimmerToUpdate].onOff == 0){
          DEBUG_PRINTLN(F("--------------"));
          DEBUG_PRINTLN(F("Leaving from ON to OFF"));
          dimmers[dimmerToUpdate].levelOld = dimmers[dimmerToUpdate].level;
          fadeToLevelDimmer(dimmerToUpdate+DIMMER_START_ADDRESS, dimmers[dimmerToUpdate].level, 0);
          dimmers[dimmerToUpdate].onOff = 0;
          EEPROM.put(dimmers[dimmerToUpdate].eepromAddressOnOff, dimmers[dimmerToUpdate].onOff);  
          dimmers[dimmerToUpdate].state = DIM_OFF;
          dimmers[dimmerToUpdate].level = dimmers[dimmerToUpdate].levelOld; //Rewriting value that was before fadeout 
          sendUpdateToController(dimmerToUpdate + DIMMER_START_ADDRESS);
        }
      break;

      case DIM_WAIT:
        dimmers[dimmerToUpdate].stateOld = dimmers[dimmerToUpdate].state;
        
        if (dimmers[dimmerToUpdate].buttonFlag == true && dimmers[dimmerToUpdate].buttonNew == 1){  //Wait for flag and rising edge          
          DEBUG_PRINTLN(F("Button pressed"));
          DEBUG_PRINTLN(F("--------------"));
          DEBUG_PRINTLN(F("Leaving from WAIT to FADE"));
          dimmers[dimmerToUpdate].state = DIM_FADE;
          dimmers[dimmerToUpdate].buttonFlag = false;  //Must be placed last for timing
        }
      
        if(dimmers[dimmerToUpdate].onOff == 0 || (millis() - dimmers[dimmerToUpdate].eventButtonTime > BUTTON_TIME_OUT_2CLICK )){
          DEBUG_PRINTLN(F("--------------"));
          DEBUG_PRINTLN(F("Leaving from WAIT to OFF"));
          dimmers[dimmerToUpdate].levelOld = dimmers[dimmerToUpdate].level;  //Storing value for use before leaving state
          fadeToLevelDimmer(dimmerToUpdate+DIMMER_START_ADDRESS, dimmers[dimmerToUpdate].level, 0);
          //fadeToLevel(dimmers[dimmerToUpdate].level, 0);
          dimmers[dimmerToUpdate].onOff = 0;
          EEPROM.put(dimmers[dimmerToUpdate].eepromAddressOnOff, dimmers[dimmerToUpdate].onOff);  
          dimmers[dimmerToUpdate].state = DIM_OFF;
          dimmers[dimmerToUpdate].level = dimmers[dimmerToUpdate].levelOld; //Restoring to old value
          sendUpdateToController(dimmerToUpdate + DIMMER_START_ADDRESS);
        }
      break;

      case DIM_FADE:
      
          if(dimmers[dimmerToUpdate].state!=dimmers[dimmerToUpdate].stateOld){
            dimmers[dimmerToUpdate].stateOld = dimmers[dimmerToUpdate].state;
            dimmers[dimmerToUpdate].levelOld = dimmers[dimmerToUpdate].level;
            dimmers[dimmerToUpdate].fadeStep = 1; //Start with fade up 
          }else{
            if(dimmers[dimmerToUpdate].level >= 100){
              dimmers[dimmerToUpdate].level = 100;
              dimmers[dimmerToUpdate].fadeStep = -1;
            }else if(dimmers[dimmerToUpdate].level <= 0){
              dimmers[dimmerToUpdate].level = 0;
              dimmers[dimmerToUpdate].fadeStep = 1; 
            }
            dimmers[dimmerToUpdate].level += dimmers[dimmerToUpdate].fadeStep;
            DEBUG_PRINTLN(dimmers[dimmerToUpdate].level);
            delay(DIM_FADE_DELAY);
          }
          
          if (dimmers[dimmerToUpdate].buttonFlag == true && dimmers[dimmerToUpdate].buttonNew == 1){ //Wait for flag and rising edge
            DEBUG_PRINTLN(F("--------------"));
            DEBUG_PRINTLN(F("Leaving from FADE to ON, updating level"));
            dimmers[dimmerToUpdate].state = DIM_ON;
            dimmers[dimmerToUpdate].levelNew = dimmers[dimmerToUpdate].level;
            EEPROM.put(dimmers[dimmerToUpdate].eepromAddressLevel, dimmers[dimmerToUpdate].level); 
            sendUpdateToController(dimmerToUpdate + DIMMER_START_ADDRESS);
            dimmers[dimmerToUpdate].buttonFlag = false;  //Must be placed last for timing
          } 
          else if(millis() - dimmers[dimmerToUpdate].eventButtonTime > BUTTON_TIME_OUT_FADE){
            DEBUG_PRINTLN(F("--------------"));
            DEBUG_PRINTLN(F("Leaving from FADE to ON"));         
            fadeToLevelDimmer(dimmerToUpdate+DIMMER_START_ADDRESS, dimmers[dimmerToUpdate].level, dimmers[dimmerToUpdate].levelOld);
            //fadeToLevel(dimmers[dimmerToUpdate].level, dimmers[dimmerToUpdate].levelOld);
            dimmers[dimmerToUpdate].level = dimmers[dimmerToUpdate].levelOld;
            dimmers[dimmerToUpdate].levelNew = dimmers[dimmerToUpdate].level;
            dimmers[dimmerToUpdate].state = DIM_ON;
          }

          if(dimmers[dimmerToUpdate].onOff == 0){
            DEBUG_PRINTLN(F("--------------"));
            DEBUG_PRINTLN(F("Leaving from ON to OFF"));
            fadeToLevelDimmer(dimmerToUpdate+DIMMER_START_ADDRESS, dimmers[dimmerToUpdate].level, 0);
            //fadeToLevel(dimmers[dimmerToUpdate].level, 0);
            dimmers[dimmerToUpdate].levelNew = dimmers[dimmerToUpdate].level; 
            dimmers[dimmerToUpdate].onOff = 0;
            EEPROM.put(dimmers[dimmerToUpdate].eepromAddressOnOff, dimmers[dimmerToUpdate].onOff);
            dimmers[dimmerToUpdate].state = DIM_OFF;
            dimmers[dimmerToUpdate].level = dimmers[dimmerToUpdate].levelOld;   //Restoring value
            sendUpdateToController(dimmerToUpdate + DIMMER_START_ADDRESS);
          }
      break;
      
    }

}





/*------------------------------------------------------------------*/
/*Incoming messages                                                 */
/*                                                                  */
/*------------------------------------------------------------------*/
void incomingMessage(const MyMessage &message) {
  int sensorNumber = 499;

  sensorNumber = message.sensor;
  DEBUG_PRINTLN(F("---------------"));
  DEBUG_PRINT(F("Message received, Sensor: "));
  DEBUG_PRINTLN(sensorNumber);
  DEBUG_PRINT(F("Message type: "));
  DEBUG_PRINTLN(message.type);

  
  if (message.type == V_LIGHT){
    int requestedState = atoi( message.data );
    
    DEBUG_PRINT(F("V_LIGHT, Value: "));
    DEBUG_PRINTLN(requestedState);

    if(sensorNumber == DIMMER1){
      dimmers[DIMMER1-1].onOff = requestedState;
    }
    if(sensorNumber == DIMMER2){
      dimmers[DIMMER2-1].onOff = requestedState;
    }    
  }
  
  if (message.type == V_DIMMER) {
    //  Retrieve the power or dim level from the incoming request message
    int requestedLevel = atoi( message.data );
    
    DEBUG_PRINT(F("V_DIMMER, Level: "));
    DEBUG_PRINTLN(requestedLevel);
    
    // Clip incoming level to valid range of 0 to 100
    requestedLevel = requestedLevel > 100 ? 100 : requestedLevel;
    requestedLevel = requestedLevel < 0   ? 0   : requestedLevel;
    
    if(sensorNumber == DIMMER1){
      dimmers[DIMMER1-1].levelNew = requestedLevel;
    }
    if(sensorNumber == DIMMER2){
      dimmers[DIMMER2-1].levelNew = requestedLevel;
    }
  }
  DEBUG_PRINTLN(F("End of message"));
  DEBUG_PRINTLN(F("---------------"));
}



/*------------------------------------------------------------------*/
/*This method provides a graceful fade up/down effect               */
/*                                                                  */
/*------------------------------------------------------------------*/
/*
int fadeToLevel(int currentLevel, int toLevel ) {
  Serial.print("toLevel Value: ");
  Serial.println(toLevel);
  int delta = ( currentLevel - toLevel ) < 0 ? 1 : -1;
  Serial.print("delta Value: ");
  Serial.println(delta);
  while ( currentLevel != toLevel ) {
    currentLevel += delta;
    Serial.println(currentLevel);
    delay( DIM_FADE_DELAY );
  }
  return toLevel;
}
*/

/*------------------------------------------------------------------*/
/*This method provides a graceful fade up/down effect               */
/*                                                                  */
/*------------------------------------------------------------------*/
int fadeToLevelDimmer(int dimmerToFade, int currentLevel, int toLevel ) {
  dimmerToFade = dimmerToFade-DIMMER_START_ADDRESS;
  
  DEBUG_PRINT(F("toLevel: "));
  DEBUG_PRINTLN(toLevel);
  
  int delta = ( currentLevel - toLevel ) < 0 ? 1 : -1;
  //Serial.print("delta Value: "); Serial.println(delta);
  while ( currentLevel != toLevel ) {
    currentLevel += delta;
    dimmers[dimmerToFade].level = currentLevel;
    DEBUG_PRINTLN(dimmers[dimmerToFade].level);  
    delay( DIM_FADE_DELAY );
  }
  return currentLevel;
}

/*------------------------------------------------------------------*/
/*Default parameters                                                */
/*                                                                  */
/*------------------------------------------------------------------*/
void WriteDefaultParametersEEPROM(){
  //Writes default params to EEPROM at first startUp/reset and there after reads the values from EEPROM
  if(EEPROM.read(EEPROM_PARAM_STORED) != 1){   
    DEBUG_PRINTLN(F("EEPROM empty, writing def values"));
    EEPROM.put(EEPROM_ADDRESS_DIM1_LEVEL, DIM_LEVEL_DEFAULT);
    EEPROM.put(EEPROM_ADDRESS_DIM1_ONOFF, LIGHT_OFF);
    EEPROM.put(EEPROM_ADDRESS_DIM2_LEVEL, DIM_LEVEL_DEFAULT);
    EEPROM.put(EEPROM_ADDRESS_DIM2_ONOFF, LIGHT_OFF);
    EEPROM.put(EEPROM_PARAM_STORED, 1);   //Marking in EEPROM that default params are written 
    DEBUG_PRINTLN(F("EEPROM written"));
  }else{
    DEBUG_PRINTLN(F("EEPROM Contains data nothing to write"));
  }
}
/*------------------------------------------------------------------*/


/*------------------------------------------------------------------*/
/*Clear EEprom                                                      */
/*                                                                  */
/*------------------------------------------------------------------*/
void clearEeprom(){
  DEBUG_PRINTLN(F("EEPROM clear initiated"));
  /***
    Iterate through each byte of the EEPROM storage.

    Larger AVR processors have larger EEPROM sizes, E.g:
    - Arduno Duemilanove: 512b EEPROM storage.
    - Arduino Uno:        1kb EEPROM storage.
    - Arduino Mega:       4kb EEPROM storage.

    Rather than hard-coding the length, you should use the pre-provided length function.
    This will make your code portable to all AVR processors.
  ***/

  for (int i = 0 ; i < EEPROM.length() ; i++) {
    EEPROM.write(i, 255);
  }
  DEBUG_PRINTLN(F("EEPROM cleared "));
}
